#!/usr/bin/env python
"""
sorno_alarm.py


   Copyright 2014 Heung Ming Tai

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import argparse
import time

from Tkinter import *
import tkFont

class Alarm(object):
    def __init__(self, use_pop_up=False):
        """
        use_pop_up -- By default, the alarm uses the system bell. This option
            changes it to use a UI pop-up instead.
        """
        self.use_pop_up = use_pop_up

    def countdown(self, seconds):
        print("Current time is %s" % time.asctime())
        print("Sleep for %s seconds" % seconds)
        time.sleep(seconds)
        print("Alarm goes off!")

        if self.use_pop_up:
            root = Tk()
            custom_font = tkFont.Font(family="Helvetica", size=30)
            root.tkraise()
            w = Label(
                root,
                text="Alarm goes off!"
                ,
                font=custom_font,
                fg="red",
            )
            w.pack()
            root.mainloop()
        else:
            # System bell
            print("\a")
            raw_input("Press enter to continue...")

class AlarmApp(object):
    def __init__(self, alarm):
        self.alarm = alarm

    def continuous_alarm(self, seconds):
        while True:
            self.alarm.countdown(seconds)


def _main():
    description = """
A console alarm which uses the system bell as the alarm bell by default. You set how
many seconds before the alarm goes off, not an absolute time in the future.
After you respond to the bell (e.g. please "Enter" in the console after the
system bell rings), it restarts the alarm and will ring again after your
specified time. Use control-c to exit the alarm completely.

Synopsis:
sorno_alarm.py <number of seconds before the alarm rings it bell>
    """

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("seconds", type=int)
    parser.add_argument(
        "--use-pop-up",
        action="store_true",
        help="By default, the alarm uses the system bell."
        " This option changes it to use a UI pop-up instead.",
    )

    args = parser.parse_args()

    alarm = Alarm(use_pop_up=args.use_pop_up)

    app = AlarmApp(alarm)
    app.continuous_alarm(args.seconds)

if __name__ == '__main__':
    _main()



#!/usr/bin/env python
"""Downloads all items from all links from a URL.

Usually, you want to use --regex to filter out what you need. Using --dry-out
to tune your regex before the actual downloading of course.

Example:

    Let say there is a webpage with url http://content.com that has a lot of
    links to http://good-stuff/stuff1.zip, http://good-stuff/stuff2.zip, etc.
    You should do this first to see if you get all the links you want:

        sorno_download_all.py --dry-run --regex "stuff.*zip" \\
            "http://content.com"

    Then run the actual command to download them.

        sorno_download_all.py --regex "stuff.*zip" "http://content.com"


    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import re
import sys

from lxml import html
import requests
from sorno import webutil
from sorno import loggingutil


_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()


class DownloadAllApp(object):
    def __init__(
        self,
        url,
        regex,
        out_dir,
        dry_run=False,
    ):
        self.url = url
        self.regex = re.compile(regex)
        self.out_dir = out_dir
        self.dry_run = dry_run

    def run(self):
        _LOG.info("Fetch links from url: %s", self.url)
        header = {
            # most common user-agent
            'user-agent': 'Mozilla/5.0 ;Windows NT 6.1; WOW64; Trident/7.0; rv:11.0; like Gecko',
        }
        page = requests.get(self.url, headers=header)
        tree = html.fromstring(page.text)
        tree.make_links_absolute(self.url, resolve_base_href=True)

        elements_links = []
        for element, attribute, link, pos in tree.iterlinks():
            if self.regex.search(link):
                elements_links.append((element, link))
            else:
                _LOG.debug("Skip [%s]", link)

        num_links = len(elements_links)
        for i, (element, link) in enumerate(elements_links):
            if self.regex.search(link):
                _LOG.info(
                    "(%s/%s) %s (%s)",
                    i + 1,
                    num_links,
                    element.text,
                    link,
                )
                new_filepath = os.path.join(
                    self.out_dir,
                    webutil.unquote_url(os.path.basename(link)),
                )

                if os.path.exists(new_filepath):
                    _LOG.info(
                        "Skip [%s] since [%s] already exists",
                        link,
                        new_filepath,
                    )
                    continue
                _LOG.info("Download to [%s]", new_filepath)
                if not self.dry_run:
                    tmp_filepath = new_filepath + ".tmp"
                    webutil.download_file(link, tmp_filepath)
                    os.rename(tmp_filepath, new_filepath)


def parse_args(cmd_args):
    description = """
        Downloads all items from all links from a URL.
    """
    parser = argparse.ArgumentParser(
        description=description,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--dry-run",
        action="store_true",
    )
    parser.add_argument(
        "--out-dir",
        help="The directory to store downloaded files,"
            + " default to \"%(default)s\".",
        default=".",
    )
    parser.add_argument(
        "--regex",
        default=".*",
        help="Only download links with matched regex",
    )
    parser.add_argument("url")

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = loggingutil.create_plain_logger("PLAIN")

    app = DownloadAllApp(
        args.url,
        args.regex,
        args.out_dir,
        dry_run=args.dry_run,
    )
    app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""sorno_gcloud_pubsub_demo.py demos the use of Google Cloud Pub/Sub.

The script can be run as a publisher or a subscriber for a Pub/Sub topic.

You need to get the json credentials file before using this script. See https://developers.google.com/identity/protocols/application-default-credentials#howtheywork.

Quickstart:

    To run as a publisher:

        sorno_gcloud_pubsub_demo.py --google-json-credentials <your-json-credentials-file> --publisher

    To run as a subscriber:

        sorno_gcloud_pubsub_demo.py --google-json-credentials <your-json-credentials-file>  --subscriber

Reference: https://cloud.google.com/pubsub/configure

    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import base64
import json
import logging
import os

import sys

import httplib2

from apiclient import discovery
from oauth2client import client as oauth2client
import googleapiclient

from sorno import loggingutil
from sorno import datetimeutil

PUBSUB_SCOPES = ['https://www.googleapis.com/auth/pubsub']
_log = logging.getLogger()


def create_pubsub_client(http=None):
    """
    https://developers.google.com/identity/protocols/application-default-credentials#howtheywork
    """
    credentials = oauth2client.GoogleCredentials.get_application_default()
    if credentials.create_scoped_required():
        credentials = credentials.create_scoped(PUBSUB_SCOPES)
    if not http:
        http = httplib2.Http()
    credentials.authorize(http)

    return discovery.build('pubsub', 'v1', http=http)

class App(object):
    def __init__(self, args):
        self.args = args

    def run(self):
        if self.args.google_json_credentials:
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = (
                self.args.google_json_credentials
            )

        if self.args.publisher:
            self.run_as_publisher()
        elif self.args.subscriber:
            self.run_as_subscriber()

        return 0

    def run_as_publisher(self):
        _log.info("I am a publisher")

        client = create_pubsub_client()

        project_id = self._get_project_id()
        if not project_id:
            _log.error("Cannot find project_id, please specify --project-id")
            return
        _log.info("Project id: %s", project_id)

        full_topic_name = self._get_topic_name(project_id)
        _log.info("Full topic name: %s", full_topic_name)

        topic = None
        _log.info("Find or create topic %s...", full_topic_name)
        try:
            topic = client.projects().topics().get(topic=full_topic_name).execute()
        except googleapiclient.errors.HttpError as error:
            # error probably means the topic does not exist
            _log.debug("Error: %s", error)

        if not topic:
            topic = client.projects().topics().create(
                    name=full_topic_name,
                    body={}).execute()
            _log.info("Created: %s", topic.get('name'))

        _log.info("Topic resource: %s", topic)

        if self.args.publish_hello_message:
            message = "Hello from sorno_gcloud_pubsub_demo.py"

            message_body = {
                'messages': [
                    {'data': base64.b64encode(message)},
                ]
            }
            _log.info("Publishing message: %s", message_body)

            resp = client.projects().topics().publish(
                topic=full_topic_name, body=message_body). execute()

            _log.info("Response: %s", resp)

    def run_as_subscriber(self):
        _log.info("I am a subscriber")

        client = create_pubsub_client()

        project_id = self._get_project_id()
        if not project_id:
            _log.error("Cannot find project_id, please specify --project-id")
            return
        _log.info("Project id: %s", project_id)

        full_topic_name = self._get_topic_name(project_id)
        _log.info("Full topic name: %s", full_topic_name)

        full_subscription_name = self._get_subscription_name(project_id)
        _log.info("Full subscription name: %s", full_subscription_name)

        # Create a POST body for the Pub/Sub request
        subscription_body = {
            # The name of the topic from which this subscription receives messages
            'topic': full_topic_name,
            # Only needed if you are using push delivery
            # 'pushConfig': {
                # 'pushEndpoint': push_endpoint
            # }
        }

        subscription = None
        _log.info("Find or create subscription %s...", full_subscription_name)
        try:
            subscription = client.projects().subscriptions().get(
                subscription=full_subscription_name
            ).execute()
        except googleapiclient.errors.HttpError as error:
            # error probably means the subscription does not exist
            _log.debug("Error: %s", error)

        if not subscription:
            subscription = client.projects().subscriptions().create(
                name=full_subscription_name,
                body=subscription_body,
            ).execute()
            _log.info("Created: %s", subscription.get('name'))

        _log.info("Subscription resource: %s", subscription)

        # Create a POST body for the Pub/Sub request
        body = {
            # Setting ReturnImmediately to false instructs the API to wait
            # to collect the message up to the size of MaxEvents, or until
            # the timeout.
            'returnImmediately': False,
            'maxMessages': 1,
        }

        while True:
            _log.info("Pull one message...")
            try:
                resp = client.projects().subscriptions().pull(
                    subscription=full_subscription_name, body=body
                ).execute()
            except KeyboardInterrupt:
                break

            _log.info("Response: %s", resp)

            received_messages = resp.get('receivedMessages')
            if received_messages is not None:
                ack_ids = []
                for received_message in received_messages:
                    pubsub_message = received_message.get('message')
                    if pubsub_message:
                        # Process messages
                        _log.info(
                            "Message decoded: %s",
                            base64.b64decode(str(pubsub_message.get('data'))),
                        )
                        _log.info(
                            "Published local time: %s",
                            datetimeutil.guess_local_datetime(
                                pubsub_message.get('publishTime')
                            ).strftime("%Y-%m-%d %H:%M:%S"),
                        )
                        # Get the message's ack ID
                        ack_ids.append(received_message.get('ackId'))

                # Create a POST body for the acknowledge request
                ack_body = {'ackIds': ack_ids}

                # Acknowledge the message.
                _log.info("Ack the message")
                client.projects().subscriptions().acknowledge(
                    subscription=full_subscription_name, body=ack_body).execute()

    def _get_topic_name(self, project_id):
        topic_name = self.args.topic_name
        full_topic_name = "projects/{project_id}/topics/{topic_name}".format(
            **locals())
        return full_topic_name

    def _get_subscription_name(self, project_id):
        name = self.args.subscription_name
        full_subscription_name = "projects/{project_id}/subscriptions/{name}".format(
            **locals()
        )
        return full_subscription_name

    def _get_project_id(self):
        project_id = self.args.project_id
        if not project_id:
            # try to get it from credentials
            credentials_filename = oauth2client._get_environment_variable_file()
            if not credentials_filename:
                credentials_filename = oauth2client._get_well_known_file()

            if credentials_filename:
                with open(credentials_filename) as file_obj:
                    client_credentials = json.load(file_obj)
                    project_id = client_credentials['project_id']
        return project_id


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--google-json-credentials",
        help="Path to the google json credentials file. See https://developers.google.com/identity/protocols/application-default-credentials#howtheywork",
    )
    parser.add_argument(
        "--project-id",
    )
    parser.add_argument(
        "--topic-name",
        default="sorno_gcloud_pubsub_demo_topic",
    )

    #
    # publisher-specific arguments
    #

    parser.add_argument(
        "--publisher",
        action="store_true",
        help="This script is running as the publisher of a Google Cloud Pubsub topic",
    )
    parser.add_argument(
        "--skip-publisher-hello-message",
        dest="publish_hello_message",
        action="store_false",
        default=True,
    )

    #
    # subscriber-specific arguments
    #

    parser.add_argument(
        "--subscriber",
        action="store_true",
        help="This script is running as the subscriber of a Google Cloud Pubsub topic",
    )

    parser.add_argument(
        "--subscription-name",
        default="sorno_gcloud_pubsub_demo_subscription",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])
    loggingutil.setup_logger(_log, debug=args.debug)

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""sorno_feedly.py manages feeds stored in Feedly.

This script does not implement an oauth flow, so just get a developer token
from https://developer.feedly.com/v3/developer to use this script.

Quickstart:

    First, get a developer access token through
    https://developer.feedly.com/v3/developer, then set the environment
    variable SORNO_FEEDLY_ACCESS_TOKEN.

        $ export SORNO_FEEDLY_ACCESS_TOKEN='YOUR ACCESS TOKEN HERE'

    Print all categories:

        $ sorno_feedly.py categories

    Print all feeds:

        $ sorno_feedly.py categories

    Print all entries, duplicated entries, and get prompted for marking
    duplicated entries to read:

        $ sorno_feedly.py entries

    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import json
import os
import sys

import requests
import six

from sorno import consoleutil
from sorno import datetimeutil
from sorno import loggingutil


ENV_VAR_FEEDLY_ACCESS_TOKEN = "SORNO_FEEDLY_ACCESS_TOKEN"

_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class FeedlyApp(object):
    """A console application for Feedly."""
    def __init__(self):
        self.access_token = None

    def process_args(self, parser, args):
        if args.access_token:
            self.access_token = args.access_token
        else:
            self.access_token = os.getenv(ENV_VAR_FEEDLY_ACCESS_TOKEN)

        if not self.access_token:
            # find the action of the access_token
            raise argparse.ArgumentError(
                self._find_action_from_parser(parser, "--access-token"),
                "Either specify your access token through --access-token"
                    " or set the environment variable " +
                    ENV_VAR_FEEDLY_ACCESS_TOKEN,
            )

    def _find_action_from_parser(self, parser, *option):
        s = "/".join(option)
        for action in parser._actions:
            if "--access-token" == argparse._get_action_name(action):
                return action
        return None

    def categories_action(self, args):
        """Prints out all the categories"""
        categories = self.get_categories()
        categories.sort(key=lambda c: c['label'].lower())
        for category in categories:
            print("Label:", category['label'])
            print("Id:", category['id'])
            print("Description:", category.get('description'))
            print("")
        return 0

    def get_categories(self):
        endpoint = self._get_endpoint("/categories")
        headers = self._get_headers()
        resp = requests.get(endpoint, headers=headers)
        _log.debug("Response: %s", resp.__dict__)

        return json.loads(resp.text)

    def feeds_action(self, args):
        self.feeds()
        return 0

    def feeds(self):
        """Prints out all the feeds"""
        endpoint = self._get_endpoint("/subscriptions")
        headers = self._get_headers()
        resp = requests.get(endpoint, headers=headers)
        _log.debug("Response: %s", resp.__dict__)

        result = json.loads(resp.text)
        for feed in result:
            id_line = "Id: %s" % feed['id']
            if "state" in feed:
                id_line += " (State: %s)" % feed["state"]
            print(id_line)
            print("Website:", feed.get('website'))
            print(
                "Categories:",
                ", ".join([c['label'] for c in feed['categories']]),
            )
            print("")

    def entries_action(self, args):
        self.entries(do_cleanup=args.do_cleanup)
        return 0

    def entries(self, do_cleanup=False):
        """Prints out all the entries for all the feeds"""
        _log.debug("Get all the category ids first")
        categories = self.get_categories()

        duplicated_entries = []
        seen_fingerprints = set()
        total_num_entries = 0

        endpoint = self._get_endpoint("/streams/contents")
        for category in categories:
            headers = self._get_headers()
            params = {
                'streamId': category['id'],
                'count': 1000,  # max is 1000
                'unreadOnly': "true"
            }
            resp = requests.get(endpoint, headers=headers, params=params)
            _log.debug("Response: %s", resp.__dict__)

            content = json.loads(resp.text)
            #from sorno import debuggingutil
            #debuggingutil.ipython_here()

            title = None
            if 'title' in content:
                title = content['title']
            else:
                title = content['id']
                if '/' in title:
                    title = title.split('/')[-1]

            print("Title:", title)
            updated = datetimeutil.timestamp_to_local_datetime(
                content['updated'] / 1000
            )
            print("Updated:", updated.strftime("%Y/%m/%d %H:%M"))
            print("Id:", content['id'])
            if 'items' in content:
                num_entries = len(content['items'])
                total_num_entries += num_entries
                for i, entry in enumerate(content['items']):
                    print(" " * 4 + "(%d/%d)" % (i + 1, num_entries))
                    self._print_entry(entry, indent=" " * 4)
                    print("")

                    if entry['fingerprint'] in seen_fingerprints:
                        duplicated_entries.append(entry)
                    else:
                        seen_fingerprints.add(entry['fingerprint'])

        print("\nTotal of %d entries\n" % total_num_entries)

        print("Duplicated entries:")
        num_entries = len(duplicated_entries)
        for i, duplicated_entry in enumerate(duplicated_entries):
            print(" " * 4 + "(%d/%d)" % (i + 1, num_entries))
            self._print_entry(duplicated_entry, indent=" " * 4)
            print("")

        if num_entries == 0:
            print("None")
        elif do_cleanup:
            ans = consoleutil.confirm(
                "Mark %d duplicated entries as read?" % num_entries)
            if ans:
                resp = self.mark_entries_as_read(
                    [entry['id'] for entry in duplicated_entries]
                )

                if resp.status_code == 200:
                    print("Done")
                else:
                    print(resp.status_code, resp.reason)

    def mark_entries_as_read(self, entry_ids):
        endpoint = self._get_endpoint("/markers")
        headers = self._get_headers_for_json_input()
        params = {
            'type': "entries",
            'entryIds': entry_ids,
            'action': "markAsRead",
        }
        resp = requests.post(endpoint, headers=headers, data=json.dumps(params))
        _log.debug("Response: %s", resp.__dict__)

        return resp

    def _print_entry(self, entry, indent=""):
        print(indent + "Title:", entry['title'].encode('utf8'))
        if 'updated' not in entry:
            entry['updated'] = entry['published']

        updated = datetimeutil.timestamp_to_local_datetime(
            entry['updated'] / 1000
        )
        print(
            indent + "Updated:",
            updated.strftime("%Y/%m/%d %H:%M"),
        )
        published = datetimeutil.timestamp_to_local_datetime(
            entry['published'] / 1000
        )
        print(
            indent + "Published:",
            published.strftime("%Y/%m/%d %H:%M"),
        )
        crawled = datetimeutil.timestamp_to_local_datetime(
            entry['crawled'] / 1000
        )
        print(
            indent + "Crawled:",
            crawled.strftime("%Y/%m/%d %H:%M"),
        )
        origin = self._get_null_safe(entry, 'origin', 'title')
        if origin:
            origin = origin.encode("utf8")
        print(indent + "Origin:", origin)
        print(
            indent + "Source:",
            self._get_null_safe(entry, 'alternate', 'href'),
        )
        print(indent + "Fingerprint:", entry['fingerprint'])
        print(indent + "Id:", entry['id'])

    def _get_endpoint(self, path):
        return "https://cloud.feedly.com/v3" + path

    def _get_headers(self):
        return {'Authorization': 'OAuth ' + self.access_token}

    def _get_headers_for_json_input(self):
        headers = self._get_headers()
        headers['content-type'] = "application/json"
        return headers

    def _get_null_safe(self, mixed, *keys):
        val = mixed
        for key in keys:
            if val is None:
                return val

            if type(val) is dict:
                val = val.get(key)
            elif type(val) is list:
                if type(key) in six.integer_types:
                    if key < len(val):
                        val = val[key]
                    else:
                        val = None
                else:
                    new_val = None
                    for item in val:
                        new_val = self._get_null_safe(item, key)
                        if new_val is not None:
                            break
                    val = new_val

        return val


def parse_args(app_obj, cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    parser.add_argument(
        "--access-token",
        help="Feedly access token. You can get a developer access token from"
            " https://developer.feedly.com/v3/developer/. You have to specify"
            " this option or set the environment variable " +
            ENV_VAR_FEEDLY_ACCESS_TOKEN,
    )

    subparsers = parser.add_subparsers(
        title="Subcommands",
        description="Some description for subcommands",
    )

    parser_categories = subparsers.add_parser(
        "categories",
        help="Print out all categories",
        description="Print out all categories",
    )
    parser_categories.set_defaults(func=app_obj.categories_action)

    parser_feeds = subparsers.add_parser(
        "feeds",
        help="Print out all feeds",
        description="Print out all feeds",
    )
    parser_feeds.set_defaults(func=app_obj.feeds_action)

    parser_entries = subparsers.add_parser(
        "entries",
        help="Print out all entries for each category",
        description="Print out all entries for each category",
    )
    parser_entries.add_argument(
        "--skip-cleanup",
        action="store_false",
        default=True,
        dest="do_cleanup",
        help="Skip the prompt for marking entries as read",
    )
    parser_entries.set_defaults(func=app_obj.entries_action)

    args = parser.parse_args(cmd_args)
    return parser, args


def main():
    global _plain_logger, _plain_error_logger

    app = FeedlyApp()
    parser, args = parse_args(app, sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    if not args.debug:
        # suppress noisy modules
        _log.getChild("requests").setLevel(logging.WARN)

    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )

    app.process_args(parser, args)
    code = args.func(args)
    if code is not None:
        sys.exit(code)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""sorno_locate_git gets the remote location of a local file/directory from a
local git repository.


    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import os
import sys


class App(object):
    def __init__(self, args):
        self.args = args

    def run(self):
        loc = self.args.file_or_dir
        if not loc:
            loc = "."

        loc_path = os.path.abspath(loc)

        if os.path.isdir(loc_path):
            path = loc_path
        else:
            path = os.path.dirname(loc_path)

        # find git config file
        git_config_file = None
        # The local root directory of your local repo, e.g. /User/user1/mygit
        root_dir = None
        while path and path != "/":
            if ".git" in os.listdir(path):
                root_dir = path
                git_dir = os.path.join(path, ".git")
                if "config" in os.listdir(git_dir):
                    git_config_file = os.path.join(git_dir, "config")
                    break

            path = os.path.dirname(path)

        if not git_config_file:
            print("Cannot find git config file", file=sys.stderr)
            return 1

        prefix = "url = "
        with open(git_config_file) as file_obj:
            for line in file_obj:
                line = line.strip()
                if line.startswith(prefix):
                    # E.g. found "http://github.com/xxx/yyy" in the git config
                    # file
                    git_repo_url = line[len(prefix):]
                    # E.g. /User/user1/mygit/my/awesome/file becomes
                    # my/awesome/file
                    relative_path = loc_path[len(root_dir):].lstrip("/")
                    if relative_path and "github.com" in git_repo_url:
                        relative_path = os.path.join(
                            "tree/master",
                            relative_path,
                        )
                    remote_loc = os.path.join(
                        # take out .git from the end
                        git_repo_url[:-4],
                        relative_path,
                    )

                    remote_loc = remote_loc.rstrip("/")
                    # found it!
                    print(remote_loc)
                    return 0

        print("Cannot find the remote url", file=sys.stderr)
        return 1


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("file_or_dir", nargs="?")

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""sorno_attach_realdate.py attaches the actual time in human readable format
for timestamps found in coming lines.

Example:

    $ cat /tmp/abc
    once upon a time 1455225387 there is
    1455225387 something called blah
    and 1455225387
    then foo

    $ cat /tmp/abc |python scripts/sorno_attach_realdate.py
    once upon a time 1455225387(2016-02-11 13:16:27) there is
    1455225387(2016-02-11 13:16:27) something called blah
    and 1455225387(2016-02-11 13:16:27)
    then foo


    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import fileinput
import re
import sys

from sorno import datetimeutil


_datetime_format = "%Y-%m-%d %H:%M:%S"
_non_timestamp_chars_regex = re.compile("[^-_0-9a-zA-Z:]")


class AttachRealDateApp(object):
    def __init__(self, args):
        self.args = args

    def run(self):
        for line in fileinput.input([]):
            sys.stdout.write(self._process(line))
        return 0

    def _process(self, line):
        newline = datetimeutil.TIMESTAMP_REGEX.sub(self._repl, line)
        if newline == line:
            newline = self._aggressive_process(line)

        return newline

    def _aggressive_process(self, line):
        words = []
        for word in line.split(' '):
            # We only parse date time strings, so at least
            # YYYYMMDDHHmmss 14 characters
            if len(word) >= 14:
                w = _non_timestamp_chars_regex.sub("", word)
                if len(w) >= 14:
                    try:
                        dt = datetimeutil.guess_local_datetime(w)
                        # We want to attach a local datetime before trailing
                        # newlines or other whitespaces for this word
                        m = re.match(r"(.*)(\s+)", word)
                        if m:
                            word = m.group(1)
                            trailing_spaces = m.group(2)
                        else:
                            trailing_spaces = ""

                        word = "%s(%s)%s" % (
                            word,
                            dt.strftime(self.args.datetime_format),
                            trailing_spaces,
                        )
                    except ValueError as ex:
                        pass

            words.append(word)

        return " ".join(words)

    def _repl(self, match):
        potential_ts = match.group()
        num = int(potential_ts)
        try:
            dt, unit = datetimeutil.number_to_local_datetime(num)
        except ValueError:
            return potential_ts

        return "%s(%s)" % (
            potential_ts,
            dt.strftime(self.args.datetime_format),
        )


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--datetime-format",
        help="The string to format datetime. Use the formats"
        " supported by strftime. Default: \"%(default)s\".",
        default=_datetime_format,
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = AttachRealDateApp(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Join the malls information in different csv files.

The first line of each csv file should be the headers. One of the header should
be "Name".

Sample run:
    sorno_join_malls_info_in_csv.py --columns-kept-last "Total Mall Store GLA" *.csv

    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import csv
import logging
import os
import re
import string
import sys

from sorno import loggingutil


_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class App(object):
    """A console application to do work"""
    def __init__(self, args):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args

    def run(self):
        filepaths = sorted(
            self.args.files,
            reverse=True,
            key=lambda k: os.path.basename(k),
        )
        mall_names = []
        mall_infos = {}

        is_last_file = True
        num_of_placeholders = 0
        num_of_placeholders_for_next_row = 0
        headers = ["Name"]
        if self.args.columns_kept_last:
            for col in sorted(self.args.columns_kept_last):
                headers.append(col)

        for filepath in filepaths:
            num_str = self.get_number_string_from_s(os.path.basename(filepath))
            _log.info("Process [%s]", num_str)
            with open(filepath) as f:
                reader = csv.DictReader(row_processor(f))
                is_first_row_encountered = False
                for row in reader:
                    for_matching = self.get_mall_names_for_matching(row['Name'])

                    if for_matching in mall_infos:
                        mall_info = mall_infos[for_matching]
                    else:
                        mall_names.append(row['Name'])
                        mall_info = [row['Name']]
                        mall_infos[for_matching] = mall_info

                    if num_of_placeholders and (
                        num_of_placeholders > len(mall_info)
                    ):
                        mall_info.extend(
                            [""] * (num_of_placeholders - len(mall_info))
                        )

                    if is_last_file:
                        if self.args.columns_kept_last:
                            for col in sorted(self.args.columns_kept_last):
                                mall_info.append(row[col])
                                del row[col]
                    else:
                        self.exclude_columns_kept_last(row)

                    # populate the rest of the fields
                    del row['Name']
                    for col in sorted(row.keys()):
                        mall_info.append(row[col])
                        if not is_first_row_encountered:
                            headers.append(num_str + " " + col)

                    if not is_first_row_encountered:
                        is_first_row_encountered = True
                    num_of_placeholders_for_next_row = len(mall_info)

            num_of_placeholders = num_of_placeholders_for_next_row

            if is_last_file:
                is_last_file = False

        print("\t".join(headers))
        for mall_name in mall_names:
            for_matching = self.get_mall_names_for_matching(mall_name)
            mall_info = mall_infos[for_matching]

            print(
                "\t".join(
                    [c.replace("\n", ", ").replace(" ,", "") for c in mall_info]
                )
            )

        return 0

    def exclude_columns_kept_last(self, info):
        if self.args.columns_kept_last:
            for col in self.args.columns_kept_last:
                if col in info:
                    del info[col]

    def get_mall_names_for_matching(self, name):
        return "".join(name.split()).lower()

    def get_number_string_from_s(self, s):
        l = 0
        r = 0
        for i, c in enumerate(s):
            if c in string.digits:
                l = i
                break

        for i, c in reversed(list(enumerate(s))):
            if c in string.digits:
                r = i + 1
                break

        if l == r:
            return s

        return s[l:r]

def row_processor(unicode_csv_data):
    for line in unicode_csv_data:
        yield re.sub(
            r"\(\d+\)",
            "",
            line.decode('utf-8').replace("\u2019","'").encode('utf-8'),
        ).replace("*","")

def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--columns-kept-last",
        help="The columns which are kept only for the last file",
        action="append",
    )
    parser.add_argument(
        "files",
        nargs="+",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger, _plain_error_logger

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Gets stock quotes and other information for stock symbols.

The script can print real-time or close to real-time stock quotes, historical
quotes, and also fundamental ratios for the stock (company).

TODO:
Use google api instead, e.g. http://www.google.com/finance/info?q=nasdaq:appl


    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse

from cStringIO import StringIO

import csv
import datetime
import logging
import math
import sys
import subprocess

from bs4 import BeautifulSoup
import requests
from sorno import loggingutil
from sorno import consoleutil

_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()
_PLAIN_ERROR_LOGGER = None  # will be created in main()


class StockApp(object):
    # https://code.google.com/p/yahoo-finance-managed/wiki/enumQuoteProperty
    FIELD_HEADERS = [
        ("name", "s"),
        ("price", "l1"),
        ("date", "d1"),
        ("time", "t1"),
        ("change", "c1"),
        ("changeInPercent", "c"),
        ("lastTradeRealtime", "k1"),
        ("changeRealtime", "c6"),
        ("changePercentRealtime", "k2"),
        ("open", "o"),
        ("high", "h"),
        ("low", "g"),
        ("volume", "v"),
        ("AverageDailyVolume", "a2"),
        # ("SharesOutstanding", "j2"), shares outstanding returns non-standard
        # csv value
        ("bidRealtime", "b3"),
        ("askRealtime", "b2"),
        ("YearLow", "j"),
        ("YearHigh", "k"),
        ("DilutedEPS", "e"),
        ("EPSEstimateCurrentYear", "e7"),
        ("EPSEstimateNextYear", "e8"),
        ("PERatio", "r"),
        ("PERatioRealtime", "r2"),
        ("ShortRatio", "s7"),
        ("DividendPayDate", "r1"),
        ("ExDividendDate", "q"),
    ]

    YAHOO_STOCK_QUOTE_CSV_API = "http://download.finance.yahoo.com/d/quotes.csv"
    YAHOO_STOCK_HISTORICAL_QUOTES_API = "http://ichart.yahoo.com/table.csv"
    YAHOO_STOCK_INSIDER_PURCHASES_PAGE_TEMPLATE = (
        "http://finance.yahoo.com/q/it?s=%s+Insider+Transactions"
    )

    QUANDL_STOCK_FUNDAMENTAL_API = "http://www.quandl.com/api/v1/datasets/OFDP/DMDRN_%s_ALLFINANCIALRATIOS.csv"

    def __init__(
        self,
        stock_symbol,
        print_fundamentals=False,
        print_history=False,
        print_kd=False,
        print_rsi=False,
        print_price_quote=True,
        num_of_days_for_history=30,
        print_insider_purchases=False,
    ):
        self.stock_symbol = stock_symbol

        self.print_fundamentals = print_fundamentals
        self.print_history = print_history
        self.print_kd = print_kd
        self.print_rsi = print_rsi
        self.print_price_quote = print_price_quote

        self.num_of_days_for_history = num_of_days_for_history
        self.print_insider_purchases = print_insider_purchases
        self.day_low = None
        self.day_high = None
        self.current_quote = None

    def run(self):
        if self.print_price_quote:
            resp = requests.get(
                self.YAHOO_STOCK_QUOTE_CSV_API,
                params={
                    's': self.stock_symbol,
                    'f': "".join([h[1] for h in self.FIELD_HEADERS]),
                    'e': ".csv",
                },
            )
            _LOG.debug("URL: %s", resp.url)

            text = resp.text.strip()
            headers = [h[0] for h in self.FIELD_HEADERS]

            lines = [",".join(headers), text]
            reader = csv.DictReader(lines)
            # there is only one row in the result
            d = list(reader)[0]

            for header in headers:
                _PLAIN_LOGGER.info("%s:\t%s", header, d[header])

            self.current_quote = float(d['price'])

            if d['high'] == "N/A":
                # we are right after the trading day
                return

            self.day_high = float(d['high'])
            self.day_low = float(d['low'])

        if self.print_fundamentals:
            resp = requests.get(
                self.QUANDL_STOCK_FUNDAMENTAL_API % self.stock_symbol.upper()
            )

            text = resp.text

            reader = csv.DictReader(text.split("\n"))

            rows = list(reader)
            rows.sort(key=lambda r: r['Date'])

            consoleutil.DataPrinter(rows).print_result(
                consoleutil.DataPrinter.PRINT_STYLE_VERBOSE
            )

        if not self.print_history and self.print_kd:
            _LOG.error(
                "In order to print stochastic oscillators, historical data"
                    " data has to be enabled (--history)"
            )
            return

        if not self.print_history and self.print_rsi:
            _LOG.error(
                "In order to print RSI, historical data"
                    " data has to be enabled (--history)"
            )
            return

        if self.print_history:
            _PLAIN_LOGGER.info("")
            self.print_historical_stock_quotes()

        if self.print_insider_purchases:
            _PLAIN_LOGGER.info("")
            self.print_insider_purchase_entries()

    def print_historical_stock_quotes(self):
        today = datetime.datetime.today()
        num_of_days_for_history = self.num_of_days_for_history
        if self.print_rsi:
            # make sure we have at least 250 data points before today
            num_of_days_for_history = max(310, self.num_of_days_for_history)
        starting_date = today - datetime.timedelta(num_of_days_for_history)

        params = {
            's': self.stock_symbol,
            'a': starting_date.month - 1,
            'b': starting_date.day,
            'c': starting_date.year,
            'd': today.month-1,
            'e': today.day,
            'f': today.year,
            'g': "d",  # daily quotes
            'ignore': ".csv",
        }

        resp = requests.get(
            self.YAHOO_STOCK_HISTORICAL_QUOTES_API,
            params=params,
        )

        _LOG.info("URL: %s", resp.url)

        # The csv data came in reverse chronlogical order and with the
        # following fields:
        # Date Open High Low Close Volume Adj-Close
        data = []
        headers = None
        for line in resp.text.split("\n"):
            if line:
                if not headers:
                    headers = line.split(',')
                    continue

                values = line.split(',')
                for i in (1, 2, 3, 4, 6):
                    values[i] = float(values[i])
                # volume should be an integer
                values[5] = int(values[5])
                data.append(values)

        if not data:
            return

        if self.print_kd or self.print_rsi:
            # The csv data came in reverse chronological order, so make reverse
            # it to calculate %k and %d chronologically.
            data.reverse()

            current_row = [
                "Current",
                float("nan"),
                self.day_high,
                self.day_low,
                self.current_quote,
                float("nan"),
                float("nan"),
            ]

            if self.print_kd:
                # The csv has the following fields:
                # Date Open High Low Close Volume Adj-Close
                lowest = None
                highest = None
                all_ks = []
                for i, row in enumerate(data):
                    if i < 13:
                        row.extend([float("nan")] * 2)
                        continue

                    data_in_period = data[i - 13:i + 1]
                    lowest = min([r[3] for r in data_in_period])
                    highest = max([r[2] for r in data_in_period])
                    current_close = row[4]
                    k = self.calculate_k(current_close, lowest, highest)
                    row.append(k)
                    all_ks.append(k)
                    if len(all_ks) >= 3:
                        # calculate %d
                        row.append(int(round(sum(all_ks[-3:]) / 3.0)))
                    else:
                        row.append(float("nan"))

                # Calculate current %k and %d
                if all_ks and self.day_high:
                    lowest = min(lowest, self.day_low)
                    highest = max(highest, self.day_high)
                    k = self.calculate_k(self.current_quote, lowest, highest)
                    current_row.append(k)
                    all_ks.append(k)
                    if len(all_ks) >= 3:
                        # calculate %d
                        current_row.append(int(round(sum(all_ks[-3:]) / 3.0)))
                    else:
                        current_row(float("nan"))

                headers.extend(["%k", "%d"])

            if self.print_rsi:
                # The csv has the following fields:
                # Date Open High Low Close Volume Adj-Close
                gain = 0.0
                loss = 0.0
                avg_gain = 0
                avg_loss = 0
                for i, row in enumerate(data):
                    if i == 0:
                        row.append(float("nan"))
                        if _LOG.isEnabledFor(logging.DEBUG):
                            row.extend([avg_gain, avg_loss])

                        continue

                    diff = data[i][4] - data[i-1][4]

                    if i <= 14:
                        if diff >= 0:
                            gain += diff
                        else:
                            loss += -1 * diff

                    if i < 14:
                        row.append(float("nan"))
                        if _LOG.isEnabledFor(logging.DEBUG):
                            row.extend([gain / float(i), loss / float(i)])
                        continue

                    if i == 14:
                        # first RS
                        avg_gain = gain / 14.0
                        avg_loss = loss / 14.0
                        row.append(self.calculate_rsi(avg_gain, avg_loss))
                        if _LOG.isEnabledFor(logging.DEBUG):
                            row.append(avg_gain)
                            row.append(avg_loss)
                        continue

                    # subsequent RS
                    if diff >= 0:
                        avg_gain = (avg_gain * 13 + diff) / 14.0
                        avg_loss = avg_loss * 13 / 14.0
                    else:
                        avg_gain = avg_gain * 13 / 14.0
                        avg_loss = (avg_loss * 13 + -1 * diff) / 14.0

                    row.append(self.calculate_rsi(avg_gain, avg_loss))
                    if _LOG.isEnabledFor(logging.DEBUG):
                        row.append(avg_gain)
                        row.append(avg_loss)

                if self.day_high:
                    current_diff = self.current_quote - data[-1][4]
                    if current_diff >= 0:
                        avg_gain = (avg_gain * 13 + current_diff) / 14.0
                        avg_loss = avg_loss * 13 / 14.0
                    else:
                        avg_gain = avg_gain * 13 / 14.0
                        avg_loss = (avg_loss * 13 + -1 * current_diff) / 14.0
                    current_row.append(self.calculate_rsi(avg_gain, avg_loss))
                    if _LOG.isEnabledFor(logging.DEBUG):
                        current_row.append(avg_gain)
                        current_row.append(avg_loss)

                headers.append("RSI")
                if _LOG.isEnabledFor(logging.DEBUG):
                    headers.append("avg_gain")
                    headers.append("avg_loss")

                if num_of_days_for_history > self.num_of_days_for_history:
                    # delete unnecessary rows
                    real_starting_date = today - datetime.timedelta(
                        self.num_of_days_for_history
                    )
                    d = real_starting_date.strftime("%Y-%m-%d")
                    for i, row in enumerate(data):
                        if row[0] >= d:
                            break

                    if i:
                        del data[0:i]

            # change it back to reverse chronlogical order
            data.append(current_row)
            data.reverse()

        # Format the data before printing them out
        out = [self.format_data_row(row) for row in data]
        consoleutil.DataPrinter(
            out,
            headers=headers,
            print_func=_PLAIN_LOGGER.info
        ).print_result()

    @staticmethod
    def format_data_row(row):
        new_row = []
        # The fields are:
        # Date Open High Low Close Volume Adj-Close
        # If self.print_kd is true, add two more fields: %k and %d
        for i in range(len(row)):
            if i == 0:
                # Date
                new_row.append(row[0])
                continue

            n = row[i]
            if math.isnan(n):
                new_row.append("N/A")
                continue

            if i in (5, 7, 8):
                new_row.append(str(n))
                continue

            new_row.append("{0:.2f}".format(n))

        return new_row

    def historical_data_row_for_printing(self, row):
        return "\t".join(
            [
                "{0:.2f}".format(float(c)) if "." in c else c
                for c in row
            ]
        )

    @staticmethod
    def calculate_k(current, lowest, highest):
        return int(
            round(
                (current - lowest) / (highest - lowest) * 100
            )
        )

    @staticmethod
    def calculate_rsi(avg_gain, avg_loss):
        if avg_loss == 0:
            return 100

        rs = avg_gain / float(avg_loss)
        return 100 - 100.0 / (1.0 + rs)

    def print_insider_purchase_entries(self):
        resp = requests.get(
            self.YAHOO_STOCK_INSIDER_PURCHASES_PAGE_TEMPLATE % self.stock_symbol,
        )
        soup = BeautifulSoup(resp.text, "lxml")
        tables = soup.select("table.yfnc_tableout1")
        if len(tables) < 3:
            _PLAIN_ERROR_LOGGER.error("No insider purchases available")
            return 1

        insider_history_table = tables[2]

        headers = []
        rows = []
        for i, tr in enumerate(insider_history_table.select("tr table tr")):
            if i == 0:
                # process headers
                for th in tr.select("th"):
                    headers.append(th.get_text())
            else:
                # process data
                rows.append([td.get_text() for td in tr.select("td")])
        consoleutil.DataPrinter(
            rows,
            headers=headers,
            print_func=_PLAIN_LOGGER.info,
        ).print_result(consoleutil.DataPrinter.PRINT_STYLE_NICETABLE)


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2014")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    parser.add_argument(
        "--fundamentals",
        action="store_true",
        help="Print fundamentals data",
    )

    parser.add_argument(
        "--insider-purchases",
        action="store_true",
        help="Print insider purchases",
    )

    parser.add_argument(
        "--history",
        help="Print historical stock quotes",
        action="store_true",
    )

    parser.add_argument(
        "--no-price-quote",
        action="store_true",
        help="Not printing price quote of the stock",
    )

    parser.add_argument(
        "-n",
        "--num-of-days",
        help="Number of days to print for historical stock quotes",
        type=int,
        default=30,
    )

    parser.add_argument(
        "-k",
        "--kd-line",
        help="Print values for fast stochastic oscillators along with"
            " historical stock quotes. This option can only be used with"
            " --num-of-days and there are at least 14 historical stock quotes."
            " The period used for %%k is 14, and %%d is 3-period"
            " moving average of %%k.",
        action="store_true",
    )

    parser.add_argument(
        "-r",
        "--rsi",
        help="Print RSI with period of 14 days. Also print the RSI that"
            " includes today (so a total of 15 data points)",
        action="store_true",
    )

    parser.add_argument("stock_symbol", nargs="+")

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER, _PLAIN_ERROR_LOGGER

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = loggingutil.create_plain_logger("PLAIN")
    _PLAIN_ERROR_LOGGER = loggingutil.create_plain_logger("PLAIN_ERROR", stdout=False)

    for stock_symbol in args.stock_symbol:
        app = StockApp(
            stock_symbol,
            print_history=args.history,
            print_kd=args.kd_line,
            print_rsi=args.rsi,
            print_price_quote=not args.no_price_quote,
            print_fundamentals=args.fundamentals,
            num_of_days_for_history=args.num_of_days,
            print_insider_purchases=args.insider_purchases,
        )
        app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import os

dependency_libs = [
    "beautifulsoup4",
    "dropbox",
    "feedparser",
    "google_api_python_client",
    "humanfriendly",
    "ipython",
    "lxml",
    # https://github.com/ahupp/python-magic
    "python_magic",
    "python_dateutil",
    "requests",
    "six",
    "tzlocal",
]

script_dir = os.path.dirname(os.path.realpath(__file__))

with open("README.rst", "r") as f:
    readme_text = f.read()

setup(
    name="sorno-py-scripts",
    version="0.41.3",
    description="Herman Tai's python scripts all prefixed with \"sorno_\"",
    long_description=readme_text,
    author="Herman Tai",
    author_email="htaihm@gmail.com",
    license="APLv2",
    url="https://github.com/hermantai/sorno-py-scripts",
    download_url="https://github.com/hermantai/sorno-py-scripts/archive/master.zip",
    packages=[
        "sorno",
    ],
    scripts=[
        os.path.join("scripts", f)
        for f in os.listdir(os.path.join(script_dir, "scripts"))
        if f.endswith(".py")
    ],  # include all py scripts under the scripts directory
    requires=dependency_libs,
    install_requires=dependency_libs,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: Apache Software License",
    ],
)



"""
Utility functions for dealing with files
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import magic


def is_text_file(filepath):
    """
    Check if a file is a text file using libmagic.

    Args:
        filepath: A string. The path to the file being inspected.

    Returns:
        True if the file is a text file.
    """
    f = magic.from_file(filepath)
    return "text" in f



"""A library for math related things


Copyright 2015 Heung Ming Tai

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from collections import namedtuple


class Interval(object):
    """An interval with a starting and a ending points, open or closed.

    It's a read-only class.

    Attributes:
        start (int or float): The starting point of the interval.
        end (int or float): The ending point of the interval.
        is_start_opened (Optional[bool]): True if the starting point is open.
            It's False by default.
        is_end_opened (Optional[bool]): True if the ending point is open.
            It's False by default.
    """
    def __init__(self, start, end, is_start_opened=False, is_end_opened=False):
        self._start = start
        self._end = end
        self._is_start_opened = is_start_opened
        self._is_end_opened = is_end_opened

    @property
    def start(self):
        return self._start

    @property
    def end(self):
        return self._end

    @property
    def is_start_opened(self):
        return self._is_start_opened

    @property
    def is_end_opened(self):
        return self._is_end_opened

    def __str__(self):
        tmp = "Interval(start=%r,end=%r,is_start_opened=%r,is_end_opened=%r)"
        return tmp % (
            self._start,
            self._end,
            self._is_start_opened,
            self._is_end_opened,
        )

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        if not isinstance(other, Interval):
            return False

        return (
            self._start,
            self._end,
            self._is_start_opened,
            self._is_end_opened,
        ) == (
            other._start,
            other._end,
            other._is_start_opened,
            other._is_end_opened,
        )
